# -*- coding: utf-8 -*-

import sys
import os
import pkgutil
import threading
import pydoc

import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk


#
#
class PydocThread(threading.Thread):
    '''
    pydocを使って、モジュールのHTMLを出力するスレッド
    '''
    def __init__(self, modList, cbStart, cbProceed, cbTerminate):
        '''
        初期化
        '''        
        threading.Thread.__init__(self)

        self.modList = modList
        self.cbStart = cbStart
        self.cbProceed = cbProceed
        self.cbTerminate = cbTerminate

    def run(self):
        '''
        スレッド処理
        '''
        cnt = len(self.modList)
        
        if self.cbStart:
            self.cbStart(cnt)

        i = 0
        for mod in self.modList:
            # HTMLを作成して保存
            pydoc.writedoc(mod)

            if self.cbProceed:
                self.cbProceed(i, cnt)
            i += 1

        if self.cbTerminate:
            self.cbTerminate()

#
# globals
#
g_progressBar = None
g_startButton = None
g_modulePath = None
g_outputPath = None
g_saveCwd = None

#
# widget signal handlers
#
def show_dir_chooser(title, parent=None):
    '''
    ディレクトリ選択ダイアログを表示
    '''
    chooser = gtk.FileChooserDialog(title=title,
                                    parent=parent,
                                    action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                             gtk.STOCK_OK, gtk.RESPONSE_OK))
    chooser.set_default_response(gtk.RESPONSE_OK)
    chooser.set_position(gtk.WIN_POS_CENTER)

    response = chooser.run()
        
    dirPath = None
    if response == gtk.RESPONSE_OK:
        dirPath = chooser.get_filename()
            
    chooser.destroy()
    return dirPath

def update_start_button():
    '''
    モジュールディレクトリと出力ディレクトリの両方が
    選択されていたら、スタートボタンを有効に。
    '''
    global g_modulePath
    global g_outputPath
    global g_startButton
    
    if (g_modulePath == None or
        g_outputPath == None or
        g_startButton == None):
        return
    
    flag = (len(g_modulePath.get_text()) > 0 and
            len(g_outputPath.get_text()) > 0)
    g_startButton.set_sensitive(flag)

def on_button_modules_path(widget, event=None):
    '''
    モジュールディレクトリ選択ボタンが押された場合の処理
    '''
    global g_modulePath
    
    if g_modulePath == None:
        return
    
    folder = show_dir_chooser('Select module directory')
    if folder:
        g_modulePath.set_text(folder)
    update_start_button()

def on_button_output_path(widget, event=None):
    '''
    出力ディレクトリ選択ボタンが押された場合の処理
    '''
    global g_outputPath
    
    if g_outputPath == None:
        return
    
    folder = show_dir_chooser('Select output directory')
    if folder:
        g_outputPath.set_text(folder)
    update_start_button()

def on_pydoc_begin(cnt):
    global g_saveCwd
    global g_outputPath
    global g_progressBar

    if (g_outputPath == None or
        g_progressBar == None):
        return
    # ワーキングディレクトリの退避

    g_saveCwd = os.getcwd()

    # 出力ディレクトリをカレントワーキングディレクトリに。
    os.chdir(g_outputPath.get_text())

    gtk.gdk.threads_enter()
    g_progressBar.set_text('0/%d' % cnt)
    g_progressBar.set_fraction(0.0)
    gtk.gdk.threads_leave()

def on_pydoc_proceed(i, cnt):
    '''
    １件処理が完了するたびにスレッドから呼び出される処理
    '''
    global g_progressBar
    
    if g_progressBar == None:
        return
    
    gtk.gdk.threads_enter()
    g_progressBar.set_text('%d/%d' % (i, cnt))
    g_progressBar.set_fraction(float(i) / float(cnt))
    gtk.gdk.threads_leave()

def on_pydoc_end():
    '''
    スレッドの全ての処理が完了した際に呼び出される処理
    '''
    global g_progressBar

    if g_progressBar == None:
        return
    
    gtk.gdk.threads_enter()
    g_progressBar.set_text('-- Finished --')
    g_progressBar.set_fraction(0.0)
    gtk.gdk.threads_leave()
        
    # ワーキングディレクトリを復帰
    os.chdir(g_saveCwd)

def on_button_start(widget, event=None):
    '''
    スタートボタンが押された場合の処理。
    モジュールドキュメント作成を開始する。
    '''
    global g_modulePath
    
    # 処理対象のモジュールディレクトリが、Pythonパスに
    # 含まれていない場合には、追加    
    if g_modulePath == None:
        return
    
    modPath = g_modulePath.get_text()
    if modPath not in sys.path:
        sys.path.append(modPath)

    # モジュールディレクトリ中のモジュールを取得
    modList = [name for loader, name, ispkg in pkgutil.walk_packages([modPath])]
    thr = PydocThread(modList, on_pydoc_begin, on_pydoc_proceed, on_pydoc_end)
    thr.start()

def get_path_select_widget(frameLabel, handler):
    '''
    パス選択用のウィジェットを生成して返す。
    '''        
    frame = gtk.Frame(frameLabel)
        
    hbox = gtk.HBox()
    hbox.set_border_width(4)
    label = gtk.Label('')
    label.set_size_request(250, 18)
    hbox.pack_start(label, True, True, 4)

    bt = gtk.Button('...')
    if handler:
        bt.connect('clicked', handler)
            
    hbox.pack_end(bt, False, False, 4)
    frame.add(hbox)

    return frame, label

#
# plugin protocol functions
#

def get_author():
    '''
    プラグイン製作者名を返す
    '''
    return u'jkani4'

def get_name():
    '''
    プラグイン名称を取得
    '''
    return u'pydocUtil'

def get_description():
    '''
    プラグインについての説明文を取得
    '''
    text = u'pydocモジュールを使用して、モジュールのドキュメントを\n指定されたディレクトリに出力します。'
    return text
    
def get_version():
    '''
    プラグインバージョン（文字列）の取得
    '''
    return u'0.5.0'

def get_widgets():
    '''
    プラグインがアクティブになった際に呼び出される関数。
    プラグインのGUIを生成して返す。
    '''
    global g_modulePath
    global g_outputPath
    global g_progressBar
    global g_startButton

    vbox = gtk.VBox()
    vbox.set_border_width(5)

    # モジュールディレクトリ選択用ウィジェットの生成
    frame, label = get_path_select_widget('Module directory',
                                          on_button_modules_path)
    g_modulePath = label
    vbox.pack_start(frame, False, False, 4)

    # ドキュメント出力ディレクトリ選択用ウィジェットの生成
    frame, label = get_path_select_widget('Output directory',
                                          on_button_output_path)
    g_outputPath = label
    vbox.pack_start(frame, False, False, 4)

    # プログレスバー
    g_progressBar = gtk.ProgressBar()
    g_progressBar.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
    g_progressBar.set_size_request(-1, 12)
    vbox.pack_start(g_progressBar, False, False, 4)

    # 開始ボタン
    g_startButton = gtk.Button('Start')
    g_startButton.connect('clicked', on_button_start)
    vbox.pack_start(g_startButton, False, False, 4)

    update_start_button()
    
    return vbox

def get_settings():
    '''
    プラグインの設定を保存する際に呼び出される関数。
    プラグインのパラメータを辞書の形で返す。
    '''
    return {}

def set_settings(settings):
    '''
    プラグインの設定を復元する際に呼び出される関数。
    渡された辞書settingのデータをもとに、設定を復元する処理
    を記述する。
    '''
    pass
